import Flutter
import UIKit
import IdensicMobileSDK


public class IdensicMobileSdkPluginImpl: NSObject, FlutterPlugin {

    var channel: FlutterMethodChannel
    weak var sdk: SNSMobileSDK?

    public static func register(with registrar: FlutterPluginRegistrar) {
        
        let channel = FlutterMethodChannel(name: "sumsub.com/flutter_idensic_mobile_sdk_plugin",
                                           binaryMessenger: registrar.messenger())
        
        let instance = IdensicMobileSdkPluginImpl(channel: channel)
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    init(channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        if call.method == "dismiss" {
            self.sdk?.dismiss()
            return
        }
        
        if call.method != "onLaunchSDK" {
            result(FlutterMethodNotImplemented)
            return
        }
        
        let channel = self.channel
        
        let args = call.arguments as? Dictionary<String, Any>
        let baseUrl = args?["apiUrl"] as? String ?? ""
        let flowName = args?["flowName"] as? String ?? ""
        let accessToken = args?["accessToken"] as? String ?? ""
        let locale = args?["languageCode"] as? String
        let supportEmail = args?["supportEmail"] as? String
        let isDebug = args?["isDebug"] as? Bool ?? false
        
        let sdk = SNSMobileSDK(baseUrl: baseUrl,
                               flowName: flowName,
                               accessToken: accessToken,
                               locale: locale,
                               supportEmail: supportEmail)
        
        guard sdk.isReady else {
            result(sdk.pluginResult)
            return
        }

        self.sdk = sdk
        
        if isDebug {
            sdk.logLevel = .debug
        }
        
        sdk.tokenExpirationHandler { (onComplete) in
            channel.invokeMethod("onTokenExpiration", arguments: nil) { (newToken) in
                onComplete(newToken as? String)
            }
        }
        
        sdk.onStatusDidChange { (sdk, prevStatus) in
            channel.invokeMethod("onStatusChanged", arguments: [
                sdk.description(for: sdk.status),
                sdk.description(for: prevStatus)
            ])
        }
        
        sdk.onDidDismiss { (sdk) in
            result(sdk.pluginResult)
        }
        
        sdk.present()
    }
}

extension SNSMobileSDK {
    
    typealias Result = [String:Any]
    
    var pluginResult: Result {
        
        var result = Result()
        
        result["success"] = status != .failed
        result["status"] = description(for: status)
        
        if (status == .failed) {
            result["errorType"] = description(for: failReason)
            result["errorMsg"] = verboseStatus
        }
        
        return result
    }
}
